/*
 * This file is part of Nuts Framework.
 * Copyright(C) 2009-2012 Nuts Develop Team.
 *
 * Nuts Framework is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License any later version.
 * 
 * Nuts Framework is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Nuts Framework. If not, see <http://www.gnu.org/licenses/>.
 */
package nuts.core.dao.sql.engine;

import nuts.core.dao.sql.SqlExecutor;
import nuts.core.dao.sql.SqlManager;
import nuts.core.lang.DynamicClassLoader;

import java.sql.Connection;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;



/**
 */
public class JavaScriptSqlManager extends SqlManager {
	/**
	 * sequence
	 */
	private int sequence = 0;

	/**
	 * dynamicClassLoader
	 */
	private DynamicClassLoader dynamicClassLoader = new DynamicClassLoader();

	/**
	 * sqlParserCache
	 */
	private Map<String, Class<SqlParser>> sqlParserCache = new ConcurrentHashMap<String, Class<SqlParser>>();
	
	/**
	 * Constructor
	 * 
	 * @param connection connection
	 * @param resultSetType one of the following ResultSet constants: ResultSet.TYPE_FORWARD_ONLY,
	 *            ResultSet.TYPE_SCROLL_INSENSITIVE, or ResultSet.TYPE_SCROLL_SENSITIVE
	 * @param resultSetConcurrency one of the following ResultSet constants:
	 *            ResultSet.CONCUR_READ_ONLY or ResultSet.CONCUR_UPDATABLE
	 * @param resultSetHoldability one of the following ResultSet constants:
	 *            ResultSet.HOLD_CURSORS_OVER_COMMIT or ResultSet.CLOSE_CURSORS_AT_COMMIT
	 */
	@Override
	public SqlExecutor getExecutor(Connection connection, int resultSetType, int resultSetConcurrency,
			int resultSetHoldability) {
		JavaScriptSqlExecutor se = new JavaScriptSqlExecutor(this);
		se.setConnection(connection);
		se.setResultSetType(resultSetType);
		se.setResultSetConcurrency(resultSetConcurrency);
		se.setResultSetHoldability(resultSetHoldability);
		return se;
	}


	/**
	 * @return the sequence
	 */
	public int getSequence() {
		return ++sequence;
	}


	/**
	 * @return the dynamicClassLoader
	 */
	public DynamicClassLoader getDynamicClassLoader() {
		return dynamicClassLoader;
	}

	/**
	 * @return the sqlParserCache
	 */
	public Map<String, Class<SqlParser>> getSqlParserCache() {
		return sqlParserCache;
	}
}
